/* GenTrxData.p   GENERATE TRANSACTION TEST DATA SET */
define temp-table tt_Customer no-undo
    field in_Seq as integer
    field in_CustNo as integer
    field ch_CustName as character
    index ix_BySeq as primary unique in_Seq.
define variable lin_MaxCustSeq as integer no-undo initial 0.

define temp-table tt_Item no-undo
    field in_Seq as integer
    field in_ItemNo as integer
    field ch_ItemName as character
    field de_Price as decimal
    index ix_BySeq as primary unique in_Seq.
define variable lin_MaxItemSeq as integer no-undo initial 0.

define temp-table tt_Trx no-undo
                                                            /* unique values */
    field in_Seq as integer
    field ch_TrxID as character
    /* omitted unique decimal */
    /* omitted unique date */
    field dt_TrxTimeStamp as datetime
                                                        /* non-unique values */
    field in_Quantity as integer
    field ch_CustName as character
    field ch_ItemName as character
    field de_TrxAmount as decimal
    field da_TrxDate as date
    field dt_PostTime as datetime
    index ix_BySeq as primary unique in_Seq
    index ix_ByTrxID as unique ch_TrxID
    index ix_ByTimeStamp as unique dt_TrxTimeStamp
    .
define variable lhn_Trx as handle no-undo.
define variable lch_TrxID as character no-undo.
define variable lin_CurTrxSeq as integer no-undo initial 0.
define variable lin_MaxTrxSeq as integer no-undo initial 100.
define variable lin_CustSeq as integer no-undo.
define variable lin_ItemSeq as integer no-undo.
define variable lin_Quantity as integer no-undo.
define variable ldt_TrxTimeStamp as datetime no-undo.
define variable ldt_PostTime as datetime no-undo.
define variable lde_TrxAmount as decimal no-undo.
define variable lch_CustName as character no-undo.
define variable lch_ItemName as character no-undo.

for each Customer:
  create tt_Customer.
  assign
      lin_MaxCustSeq = lin_MaxCustSeq + 1
      tt_Customer.in_Seq = lin_MaxCustSeq
      tt_Customer.in_CustNo = Customer.CustNum
      tt_Customer.ch_CustName = Customer.Name
      .
end.
for each Item:
  create tt_Item.
  assign
      lin_MaxItemSeq = lin_MaxItemSeq + 1
      tt_Item.in_Seq = lin_MaxItemSeq
      tt_Item.in_ItemNo = Item.ItemNum
      tt_Item.ch_ItemName = Item.ItemName
      tt_Item.de_Price = Item.Price
      .
end.

repeat:
  assign
      lin_CurTrxSeq = lin_CurTrxSeq + 1
      lin_CustSeq = random(1, lin_MaxCustSeq)
      lin_ItemSeq = random(1, lin_MaxItemSeq)
      lin_Quantity = random(1, 10)
      .
  repeat:
    ldt_TrxTimeStamp = datetime( date(random(2453797, 2453827)) , random(0,86399999) ).
    if not can-find(tt_Trx where tt_Trx.dt_TrxTimeStamp EQ ldt_TrxTimeStamp) then leave.
  end.
  ldt_PostTime = datetime( date(ldt_TrxTimeStamp), 3600000 * ( int( truncate(mtime(ldt_TrxTimeStamp) / 3600000, 0) ) + 1) ).
  find tt_Customer where tt_Customer.in_Seq = lin_CustSeq.
  lch_TrxID = replace(string(tt_Customer.ch_CustName), " ", "") + replace(replace(replace(iso-date(ldt_TrxTimeStamp), "-", ""), ":", ""), ".", "").
  lch_CustName = tt_Customer.ch_CustName.
  find tt_Item where tt_Item.in_Seq = lin_ItemSeq.
  lch_ItemName = tt_Item.ch_ItemName.
  lde_TrxAmount = lin_Quantity * tt_Item.de_Price.
  create tt_Trx.
  assign
      tt_Trx.in_Seq = lin_CurTrxSeq
      tt_Trx.ch_TrxID = lch_TrxID
      tt_Trx.dt_TrxTimeStamp = ldt_TrxTimeStamp
      tt_Trx.in_Quantity = lin_Quantity
      tt_Trx.ch_CustName = lch_CustName
      tt_Trx.ch_ItemName = lch_ItemName
      tt_Trx.de_TrxAmount = lde_TrxAmount
      tt_Trx.da_TrxDate = date(ldt_TrxTimeStamp)
      tt_Trx.dt_PostTime = ldt_PostTime
      .
  if lin_CurTrxSeq GE lin_MaxTrxSeq then leave.
end.
lhn_Trx = temp-table tt_Trx:handle.
lhn_Trx:write-xml(
    "file",
    "TrxData.xml",
    true,
    "UTF-8"
    ).
lhn_Trx:write-xmlschema(
    "file",
    "TrxDataSchema.xml"
    ).
empty temp-table tt_Customer.
empty temp-table tt_Item.
empty temp-table tt_Trx.
