/* com.cintegrity.Lib.Collection.TestDatetimeObjectListMap.p */

define temp-table tt_Trx no-undo
                                                            /* unique values */
    field in_Seq as integer
    field ch_TrxID as character
    /* omitted unique decimal */
    /* omitted unique date */
    field dt_TrxTimeStamp as datetime
                                                        /* non-unique values */
    field in_Quantity as integer
    field ch_CustName as character
    field ch_ItemName as character
    field de_TrxAmount as decimal
    field da_TrxDate as date
    field dt_PostTime as datetime
    index ix_BySeq as primary unique in_Seq
    index ix_ByTimeStamp as unique dt_TrxTimeStamp
    .
define variable lhn_Trx as handle no-undo.
define variable lob_myList as com.cintegrity.Lib.Collection.DatetimeObjectListMap no-undo.
define variable ldt_PostTime as datetime no-undo.
define variable lob_CurrentTrx as com.cintegrity.Lib.Collection.Trx no-undo.
define variable lob_OldTrx as Progress.Lang.Object.
define variable lob_FirstTrx as com.cintegrity.Lib.Collection.Trx no-undo.
define variable lob_LastTrx as com.cintegrity.Lib.Collection.Trx no-undo.
define variable lch_InputFileName as character no-undo initial "com/cintegrity/Lib/Collection/TrxData.xml".
define variable lch_SchemaFileName as character no-undo initial "com/cintegrity/Lib/Collection/TrxDataSchema.xml".
define variable ldt_CurrentKey as datetime no-undo.
define variable lin_TargetCount as integer no-undo.
define variable lin_Count as integer no-undo.
define variable llg_IsOK as logical no-undo.
define variable lin_CurrentSeq as integer no-undo.

if search( lch_InputFileName ) ne ?
then lch_InputFileName = search( lch_InputFileName ).
else do:
  message "Could not find test input file " + lch_InputFileName + " - aborting" view-as alert-box.
  return.
end.

if search( lch_SchemaFileName ) ne ?
then lch_SchemaFileName = search( lch_SchemaFileName ).
else do:
  message "Could not find Schema file " + lch_SchemaFileName + " - aborting" view-as alert-box.
  return.
end.

lhn_Trx = temp-table tt_Trx:handle.
lhn_Trx:read-xml(
    "FILE",
    lch_InputFileName,
    "empty",
    lch_SchemaFileName,
    false
    ).

lob_myList = new com.cintegrity.Lib.Collection.DatetimeObjectListMap().

for each tt_Trx:
  lob_CurrentTrx = new com.cintegrity.Lib.Collection.Trx(
      input tt_Trx.in_Seq,
      input tt_Trx.ch_TrxID,
      input tt_Trx.dt_TrxTimeStamp,
      input tt_Trx.in_Quantity,
      input tt_Trx.ch_CustName,
      input tt_Trx.ch_ItemName,
      input tt_Trx.de_TrxAmount,
      input tt_Trx.da_TrxDate,
      input tt_Trx.dt_PostTime
      ).
  lob_myList:putValue(
       tt_Trx.dt_PostTime,
       lob_CurrentTrx
       ).
  if lob_FirstTrx eq ?
  then lob_FirstTrx = lob_CurrentTrx.
end.
lob_LastTrx = lob_CurrentTrx.

message "Number of objects in ListMap = " + string(lob_myList:size()) view-as alert-box.
message "Number of unique keys = " + string(lob_myList:NumberOfKeys()) view-as alert-box.
message "Is set empty? " + string(lob_myList:isEmpty()) view-as alert-box.

message "Set contains last object added = " + string(lob_myList:containsValue( lob_LastTrx )) view-as alert-box.

ldt_PostTime = lob_myList:getLastKey().
message "The Last Key = " + string(lob_myList:getLastKey()) view-as alert-box.
message "The Last PostTime = " + string(cast(lob_myList:getLastValue(), "com.cintegrity.Lib.Collection.Trx"):getPostTime()) view-as alert-box.

message "Positioned at end, then 'has next' = " + string(lob_myList:hasNextKey()) + " and 'has prev' = " + string(lob_myList:hasPrevKey() ) view-as alert-box.

repeat:
  lob_CurrentTrx = cast(lob_myList:getNextValueSameKey(), "com.cintegrity.Lib.Collection.Trx").
  if not valid-object( lob_CurrentTrx )
  then leave.
end.
lin_CurrentSeq = lob_myList:getCurrentSeq().
lob_myList:removeValue( ldt_PostTime, lin_CurrentSeq ).

message "After removing last Object, number of values = " + string(lob_myList:size()) view-as alert-box.
message "The Last Key now is = " + string(lob_myList:getLastKey()) view-as alert-box.

message "The Next to the last Key = " + string(lob_myList:getPrevKey()) view-as alert-box.
message "The Next to the last PostTime = " + string(cast(lob_myList:getPrevValue(), "com.cintegrity.Lib.Collection.Trx"):getPostTime()) view-as alert-box.

lob_CurrentTrx = cast(lob_myList:getFirstValue(), "com.cintegrity.Lib.Collection.Trx").
message "Positioned at the beginning, then 'has next' = " + string(lob_myList:hasNextKey() )  " and 'has prev' = " + string(lob_myList:hasPrevKey() ) view-as alert-box.

message "The First Key = " + string(lob_myList:getFirstKey()) view-as alert-box.
message "The First PostTime = " + string(cast(lob_myList:getFirstValue(), "com.cintegrity.Lib.Collection.Trx"):getPostTime()) view-as alert-box.

message "The Next to the first Key = " + string(lob_myList:getNextKey()) view-as alert-box.
message "The Next to the first PostTime = " + string(cast(lob_myList:getNextValue(), "com.cintegrity.Lib.Collection.Trx"):getPostTime()) view-as alert-box.

lin_TargetCount = random( 1, lob_myList:NumberOfKeys() ) - 1.
lob_OldTrx = lob_myList:getFirstValue().

do lin_Count = 1 to lin_TargetCount - 1:
  ldt_CurrentKey = cast(lob_myList:getNextValue(), "com.cintegrity.Lib.Collection.Trx"):getPostTime().
  if ldt_CurrentKey eq ?
  then do:
    message "Unexpected error searching for random record" view-as alert-box.
    return.
  end.
end.

message "The " + string(lin_TargetCount) + "th Key = " + string(ldt_CurrentKey) view-as alert-box.
message "The " + string(lin_TargetCount) + "th PostTime = " + string(cast(lob_myList:getValue( ldt_CurrentKey ), "com.cintegrity.Lib.Collection.Trx"):getPostTime()) view-as alert-box.
message "The " + string(lin_TargetCount) + "th Key has " + string(lob_myList:CountOfKey( ldt_CurrentKey )) + " different values" view-as alert-box.
message "Set contains this key = " + string(lob_myList:containsKey( ldt_CurrentKey )) view-as alert-box.
message "The current Key is " + string(lob_myList:getCurrentKey()) view-as alert-box.

lob_myList:clearAll().
message "After clearing all Objects, number of values = " + string(lob_myList:size()) view-as alert-box.
if valid-object(  lob_myList )
then delete object lob_myList.
if valid-object( lob_FirstTrx )
then delete object lob_FirstTrx.
if valid-object( lob_LastTrx )
then delete object lob_LastTrx.
if valid-object( lob_CurrentTrx )
then delete object lob_CurrentTrx.
if valid-object( lob_OldTrx )
then delete object lob_OldTrx.
assign
    lob_myList = ?
    lob_FirstTrx = ?
    lob_LastTrx = ?
    lob_CurrentTrx = ?
    lob_OldTrx = ?
    .
